@@ -36,7 +36,7 @@ impl From<ser::SerializerError> for Error {
 const ERR_INVALID_VALUE: &str = "Invalid value";
 
 #[allow(clippy::module_name_repetitions)]
-#[derive(Debug, Serialize, Clone)]
+#[derive(Debug, Serialize, Clone, Eq, PartialEq)]
 #[serde(untagged)]
 pub enum ValueOptionOwned {
     No,
@@ -87,7 +87,7 @@ impl From<Option<Value>> for ValueOptionOwned {
 }
 
 #[allow(clippy::module_name_repetitions)]
-#[derive(Debug, Serialize, Clone)]
+#[derive(Debug, Serialize, Clone, Eq, PartialEq)]
 #[serde(untagged)]
 pub enum ValueOption<'a> {
     No,
@@ -491,6 +491,166 @@ impl TryFrom<&Value> for f64 {
     }
 }
 
+impl TryFrom<Value> for u8 {
+    type Error = Error;
+    #[inline]
+    fn try_from(value: Value) -> EResult<u8> {
+        macro_rules! ngt {
+            ($n: expr, $x: ty) => {
+                if $n > std::u8::MAX as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too big: {} (max: {}",
+                        $n,
+                        std::u8::MAX
+                    )));
+                } else if $n < std::u8::MIN as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too small: {} (min: {}",
+                        $n,
+                        std::u8::MIN
+                    )));
+                } else {
+                    $n as u8
+                }
+            };
+        }
+        match value {
+            Value::F32(v) => Ok(ngt!(v, f32)),
+            Value::F64(v) => Ok(ngt!(v, f64)),
+            Value::U8(v) => Ok(v),
+            Value::U16(v) => Ok(ngt!(v, u16)),
+            Value::U32(v) => Ok(ngt!(v, u32)),
+            Value::U64(v) => Ok(ngt!(v, u64)),
+            Value::I8(v) => Ok(ngt!(v, i8)),
+            Value::I16(v) => Ok(ngt!(v, i16)),
+            Value::I32(v) => Ok(ngt!(v, i32)),
+            Value::I64(v) => Ok(ngt!(v, i64)),
+            Value::String(v) => Ok(v.parse::<u8>()?),
+            _ => Err(Error::invalid_data(ERR_INVALID_VALUE)),
+        }
+    }
+}
+
+impl TryFrom<Value> for i8 {
+    type Error = Error;
+    #[inline]
+    fn try_from(value: Value) -> EResult<i8> {
+        macro_rules! ngt {
+            ($n: expr, $x: ty) => {
+                if $n > std::i8::MAX as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too big: {} (max: {}",
+                        $n,
+                        std::i8::MAX
+                    )));
+                } else if $n < std::i8::MIN as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too small: {} (min: {}",
+                        $n,
+                        std::i8::MIN
+                    )));
+                } else {
+                    $n as i8
+                }
+            };
+        }
+        match value {
+            Value::F32(v) => Ok(ngt!(v, f32)),
+            Value::F64(v) => Ok(ngt!(v, f64)),
+            Value::U8(v) => Ok(ngt!(v, u8)),
+            Value::U16(v) => Ok(ngt!(v, u16)),
+            Value::U32(v) => Ok(ngt!(v, u32)),
+            Value::U64(v) => Ok(ngt!(v, u64)),
+            Value::I8(v) => Ok(v),
+            Value::I16(v) => Ok(ngt!(v, i16)),
+            Value::I32(v) => Ok(ngt!(v, i32)),
+            Value::I64(v) => Ok(ngt!(v, i64)),
+            Value::String(v) => Ok(v.parse::<i8>()?),
+            _ => Err(Error::invalid_data(ERR_INVALID_VALUE)),
+        }
+    }
+}
+
+impl TryFrom<Value> for u16 {
+    type Error = Error;
+    #[inline]
+    fn try_from(value: Value) -> EResult<u16> {
+        macro_rules! ngt {
+            ($n: expr, $x: ty) => {
+                if $n > std::u16::MAX as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too big: {} (max: {}",
+                        $n,
+                        std::u16::MAX
+                    )));
+                } else if $n < std::u16::MIN as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too small: {} (min: {}",
+                        $n,
+                        std::u16::MIN
+                    )));
+                } else {
+                    $n as u16
+                }
+            };
+        }
+        match value {
+            Value::F32(v) => Ok(ngt!(v, f32)),
+            Value::F64(v) => Ok(ngt!(v, f64)),
+            Value::U8(v) => Ok(u16::from(v)),
+            Value::U16(v) => Ok(v),
+            Value::U32(v) => Ok(ngt!(v, u32)),
+            Value::U64(v) => Ok(ngt!(v, u64)),
+            Value::I8(v) => Ok(ngt!(v, i8)),
+            Value::I16(v) => Ok(ngt!(v, i16)),
+            Value::I32(v) => Ok(ngt!(v, i32)),
+            Value::I64(v) => Ok(ngt!(v, i64)),
+            Value::String(v) => Ok(v.parse::<u16>()?),
+            _ => Err(Error::invalid_data(ERR_INVALID_VALUE)),
+        }
+    }
+}
+
+impl TryFrom<Value> for i16 {
+    type Error = Error;
+    #[inline]
+    fn try_from(value: Value) -> EResult<i16> {
+        macro_rules! ngt {
+            ($n: expr, $x: ty) => {
+                if $n > std::i16::MAX as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too big: {} (max: {}",
+                        $n,
+                        std::i16::MAX
+                    )));
+                } else if $n < std::i16::MIN as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too small: {} (min: {}",
+                        $n,
+                        std::i16::MIN
+                    )));
+                } else {
+                    $n as i16
+                }
+            };
+        }
+        match value {
+            Value::F32(v) => Ok(ngt!(v, f32)),
+            Value::F64(v) => Ok(ngt!(v, f64)),
+            Value::U8(v) => Ok(i16::from(v)),
+            Value::U16(v) => Ok(ngt!(v, u16)),
+            Value::U32(v) => Ok(ngt!(v, u32)),
+            Value::U64(v) => Ok(ngt!(v, u64)),
+            Value::I8(v) => Ok(i16::from(v)),
+            Value::I16(v) => Ok(v),
+            Value::I32(v) => Ok(ngt!(v, i32)),
+            Value::I64(v) => Ok(ngt!(v, i64)),
+            Value::String(v) => Ok(v.parse::<i16>()?),
+            _ => Err(Error::invalid_data(ERR_INVALID_VALUE)),
+        }
+    }
+}
+
 impl TryFrom<Value> for u32 {
     type Error = Error;
     #[inline]
@@ -735,6 +895,46 @@ impl TryFrom<Value> for i64 {
     }
 }
 
+impl TryFrom<Value> for f32 {
+    type Error = Error;
+    #[inline]
+    fn try_from(value: Value) -> EResult<f32> {
+        macro_rules! ngt {
+            ($n: expr, $x: ty) => {
+                if $n > std::f32::MAX as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too big: {} (max: {}",
+                        $n,
+                        std::f32::MAX
+                    )));
+                } else if $n < std::f32::MIN as $x {
+                    return Err(Error::invalid_data(format!(
+                        "Value too small: {} (min: {}",
+                        $n,
+                        std::f32::MIN
+                    )));
+                } else {
+                    $n as f32
+                }
+            };
+        }
+        match value {
+            Value::F32(v) => Ok(v),
+            Value::F64(v) => Ok(ngt!(v, f64)),
+            Value::U8(v) => Ok(f32::from(v)),
+            Value::U16(v) => Ok(f32::from(v)),
+            Value::U32(v) => Ok(ngt!(v, u32)),
+            Value::U64(v) => Ok(ngt!(v, u64)),
+            Value::I8(v) => Ok(ngt!(v, i8)),
+            Value::I16(v) => Ok(ngt!(v, i16)),
+            Value::I32(v) => Ok(ngt!(v, i32)),
+            Value::I64(v) => Ok(ngt!(v, i64)),
+            Value::String(v) => Ok(v.parse::<f32>()?),
+            _ => Err(Error::invalid_data(ERR_INVALID_VALUE)),
+        }
+    }
+}
+
 impl TryFrom<Value> for f64 {
     type Error = Error;
     #[inline]
